@@ -1,7 +1,7 @@
 use std::{collections::HashMap, convert::TryInto};
 
 use crate::{
-    descriptor::ty::{EnumDescriptorInner, MessageDescriptorInner},
+    descriptor::ty::{EnumDescriptorInner, ExtensionDescriptorInner, MessageDescriptorInner},
     DescriptorError,
 };
 
@@ -19,9 +19,11 @@ enum TypeKind {
 }
 
 pub(in crate::descriptor) struct TypeMap {
-    named_types: HashMap<String, TypeId>,
+    named_types: HashMap<Box<str>, TypeId>,
     messages: Vec<MessageDescriptorInner>,
     enums: Vec<EnumDescriptorInner>,
+    extensions: Vec<ExtensionDescriptorInner>,
+    extension_names: HashMap<Box<str>, usize>,
 }
 
 impl TypeMap {
@@ -48,6 +50,8 @@ impl TypeMap {
             named_types: HashMap::new(),
             messages: Vec::new(),
             enums: Vec::new(),
+            extensions: Vec::new(),
+            extension_names: HashMap::new(),
         }
     }
 
@@ -59,10 +63,16 @@ impl TypeMap {
         (0..self.enums.len()).map(|id| TypeId(TypeKind::Message, id as u32))
     }
 
+    pub fn extensions(&self) -> impl ExactSizeIterator<Item = usize> + '_ {
+        0..self.extensions.len()
+    }
+
     pub fn shrink_to_fit(&mut self) {
         self.named_types.shrink_to_fit();
         self.messages.shrink_to_fit();
         self.enums.shrink_to_fit();
+        self.extensions.shrink_to_fit();
+        self.extension_names.shrink_to_fit();
     }
 
     pub(super) fn add_message(&mut self, message: MessageDescriptorInner) -> TypeId {
@@ -82,7 +92,13 @@ impl TypeMap {
             name = &name[1..];
         }
 
-        self.named_types.insert(name.to_owned(), ty);
+        self.named_types.insert(name.into(), ty);
+    }
+
+    pub(super) fn add_extension(&mut self, field: ExtensionDescriptorInner) {
+        let index = self.extensions.len();
+        self.extension_names.insert(field.json_name.clone(), index);
+        self.extensions.push(field);
     }
 
     pub(super) fn get(&self, id: TypeId) -> Type {
@@ -112,4 +128,12 @@ impl TypeMap {
     pub(super) fn get_scalar(&self, scalar: Scalar) -> TypeId {
         TypeId(TypeKind::Scalar, scalar as u32)
     }
+
+    pub(super) fn get_extension(&self, idx: usize) -> &ExtensionDescriptorInner {
+        self.extensions.get(idx).unwrap()
+    }
+
+    pub(super) fn try_get_extension_by_name(&self, name: &str) -> Option<usize> {
+        self.extension_names.get(name).copied()
+    }
 }
