@@ -9,17 +9,29 @@ use crate::{
     DynamicMessage, FieldDescriptor, MapKey, Value,
 };
 
+use super::field::FieldDescriptorLike;
+
 impl Message for DynamicMessage {
     fn encode_raw<B>(&self, buf: &mut B)
     where
         B: BufMut,
         Self: Sized,
     {
-        for field in self.fields.values() {
+        for field in self.fields.fields.values() {
             if let Some(value) = &field.value {
                 value.encode_field(&field.desc, buf);
             }
         }
+        if let Some(extension_fields) = self.extension_fields() {
+            for field in extension_fields.fields.values() {
+                if let Some(value) = &field.value {
+                    value.encode_field(&field.desc, buf);
+                }
+            }
+        }
+        if let Some(unknown_fields) = self.unknown_fields() {
+            unknown_fields.encode_raw(buf);
+        }
     }
 
     fn merge_field<B>(
@@ -33,40 +45,54 @@ impl Message for DynamicMessage {
         B: Buf,
         Self: Sized,
     {
-        if let Some(field) = self.fields.get_mut(&number) {
-            let field_value = &mut field.value;
-            let field_desc = &field.desc;
-            field_value
-                .get_or_insert_with(|| Value::default_value_for_field(field_desc))
-                .merge_field(field_desc, wire_type, buf, ctx)
+        if let Some(field_desc) = self.desc.get_field(number) {
+            self.fields
+                .get_mut(&field_desc)
+                .value
+                .get_or_insert_with(|| Value::default_value_for_field(&field_desc))
+                .merge_field(&field_desc, wire_type, buf, ctx)?;
+            if let Some(oneof_desc) = field_desc.containing_oneof() {
+                self.clear_oneof_fields(oneof_desc, number);
+            }
+            Ok(())
         } else {
-            prost::encoding::skip_field(wire_type, number, buf, ctx)
+            self.unknown_fields_mut()
+                .merge_field(number, wire_type, buf, ctx)
         }
     }
 
     fn encoded_len(&self) -> usize {
         let mut len = 0;
-        for field in self.fields.values() {
+        for field in self.fields.fields.values() {
             if let Some(value) = &field.value {
                 len += value.encoded_len(&field.desc);
             }
         }
+        if let Some(extension_fields) = self.extension_fields() {
+            for field in extension_fields.fields.values() {
+                if let Some(value) = &field.value {
+                    len += value.encoded_len(&field.desc);
+                }
+            }
+        }
+        if let Some(unknown_fields) = self.unknown_fields() {
+            len += unknown_fields.encoded_len();
+        }
         len
     }
 
     fn clear(&mut self) {
-        for value in self.fields.values_mut() {
-            value.clear();
-        }
+        self.fields.fields.clear();
+        self.cold = None;
     }
 }
 
 impl Value {
-    fn encode_field<B>(&self, field_desc: &FieldDescriptor, buf: &mut B)
+    pub(super) fn encode_field<B>(&self, field_desc: &impl FieldDescriptorLike, buf: &mut B)
     where
         B: BufMut,
     {
-        if !field_desc.supports_presence() && self.is_default_for_field(field_desc) {
+        if !field_desc.supports_presence() && field_desc.is_default_value(self) {
             return;
         }
 
@@ -251,9 +277,9 @@ impl Value {
         }
     }
 
-    fn merge_field<B>(
+    pub(super) fn merge_field<B>(
         &mut self,
-        field_desc: &FieldDescriptor,
+        field_desc: &impl FieldDescriptorLike,
         wire_type: WireType,
         buf: &mut B,
         ctx: DecodeContext,
@@ -318,7 +344,7 @@ impl Value {
                 }
             }
             (Value::List(values), field_kind) if field_desc.is_list() => {
-                if field_desc.is_packed() && wire_type == WireType::LengthDelimited {
+                if wire_type == WireType::LengthDelimited && field_desc.is_packable() {
                     let packed_wire_type = match field_desc.kind() {
                         Kind::Double | Kind::Fixed64 | Kind::Sfixed64 => WireType::SixtyFourBit,
                         Kind::Float | Kind::Fixed32 | Kind::Sfixed32 => WireType::ThirtyTwoBit,
@@ -377,8 +403,8 @@ impl Value {
         }
     }
 
-    fn encoded_len(&self, field_desc: &FieldDescriptor) -> usize {
-        if !field_desc.supports_presence() && self.is_default_for_field(field_desc) {
+    pub(super) fn encoded_len(&self, field_desc: &impl FieldDescriptorLike) -> usize {
+        if !field_desc.supports_presence() && field_desc.is_default_value(self) {
             return 0;
         }
 
