@@ -1,33 +1,41 @@
+mod field;
 mod message;
 #[cfg(feature = "serde")]
 mod serde;
+mod unknown;
 
-use std::{
-    borrow::Cow,
-    collections::{BTreeMap, HashMap},
-};
+use std::{borrow::Cow, collections::HashMap};
 
 #[cfg(feature = "serde")]
 pub use self::serde::{DeserializeOptions, SerializeOptions};
 
-use prost::{bytes::Bytes, DecodeError, Message};
+use prost::{
+    bytes::{Buf, Bytes},
+    DecodeError, Message,
+};
 
-use crate::{descriptor::Kind, FieldDescriptor, MessageDescriptor, ReflectMessage};
+use self::{field::DynamicMessageFieldSet, unknown::UnknownFieldSet};
+use crate::{
+    descriptor::Kind, ExtensionDescriptor, FieldDescriptor, MessageDescriptor, OneofDescriptor,
+    ReflectMessage,
+};
 
 /// [`DynamicMessage`] provides encoding, decoding and reflection of a protobuf message.
 ///
 /// It wraps a [`MessageDescriptor`] and the [`Value`] for each field of the message, and implements
 /// [`Message`][`prost::Message`].
-#[derive(Debug, Clone, PartialEq)]
+#[derive(Debug, Clone)]
 pub struct DynamicMessage {
     desc: MessageDescriptor,
-    fields: BTreeMap<u32, DynamicMessageField>,
+    fields: DynamicMessageFieldSet<FieldDescriptor>,
+    // Rarely used fields are put behind a box so they can be represented by just a null pointer in the common case.
+    cold: Option<Box<DynamicMessageColdFields>>,
 }
 
-#[derive(Debug, Clone, PartialEq)]
-struct DynamicMessageField {
-    desc: FieldDescriptor,
-    value: Option<Value>,
+#[derive(Default, Debug, Clone)]
+struct DynamicMessageColdFields {
+    unknown: UnknownFieldSet,
+    extensions: DynamicMessageFieldSet<ExtensionDescriptor>,
 }
 
 /// A dynamically-typed protobuf value.
@@ -82,18 +90,27 @@ pub enum MapKey {
 }
 
 impl DynamicMessage {
-    /// Creates a new, empty instance of [`DynamicMessage`] for the message type specified by [`MessageDescriptor`].
+    /// Creates a new, empty instance of [`DynamicMessage`] for the message type specified by the [`MessageDescriptor`].
     pub fn new(desc: MessageDescriptor) -> Self {
         DynamicMessage {
-            fields: desc
-                .fields()
-                .map(|field_desc| (field_desc.number(), DynamicMessageField::new(field_desc)))
-                .collect(),
+            fields: DynamicMessageFieldSet::default(),
+            cold: None,
             desc,
         }
     }
 
-    /// Returns `true` if this message has a field set with the number `number`.
+    /// Decodes an instance of the message type specified by the [`MessageDescriptor`] from the buffer and merges it into a
+    /// new instance of [`DynamicMessage`].
+    pub fn decode<B>(desc: MessageDescriptor, buf: B) -> Result<Self, DecodeError>
+    where
+        B: Buf,
+    {
+        let mut message = DynamicMessage::new(desc);
+        message.merge(buf)?;
+        Ok(message)
+    }
+
+    /// Returns `true` if this message has the given field set.
     ///
     /// If the field type supports distinguishing whether a value has been set, such as
     /// for messages, then this method returns `true` only if a value has been set. For other
@@ -101,87 +118,162 @@ impl DynamicMessage {
     ///
     /// If this method returns `false`, then the field will not be included in the encoded bytes
     /// of this message.
-    pub fn has_field(&self, number: u32) -> bool {
-        self.fields
-            .get(&number)
-            .map_or(false, |field| field.is_populated())
+    pub fn has_field(&self, field_desc: &FieldDescriptor) -> bool {
+        self.fields.has(field_desc)
     }
 
-    /// Gets the value of the field with number `number`, or the default value if it is unset.
-    ///
-    /// If this message has no field with number `number`, `None` is returned. Otherwise this method
-    /// will always return `Some`.
-    pub fn get_field(&self, number: u32) -> Option<Cow<'_, Value>> {
-        self.fields.get(&number).map(|field| field.get())
+    /// Gets the value of the given field, or the default value if it is unset.
+    pub fn get_field(&self, field_desc: &FieldDescriptor) -> Cow<'_, Value> {
+        self.fields.get(field_desc)
     }
 
-    /// Sets the value of the field with number `number`, or the default value if it is unset.
-    ///
-    /// If no field has number `number` this method does nothing.
+    /// Sets the value of the given field.
     ///
     /// # Panics
     ///
     /// This method may panic if the value type is not compatible with the field type, as defined
     /// by [`Value::is_valid_for_field`].
-    pub fn set_field(&mut self, number: u32, value: Value) {
-        if let Some(field) = self.fields.get_mut(&number) {
-            field.set(value);
+    pub fn set_field(&mut self, field_desc: &FieldDescriptor, value: Value) {
+        self.fields.set(field_desc, value);
+        if let Some(oneof_desc) = field_desc.containing_oneof() {
+            self.clear_oneof_fields(oneof_desc, field_desc.number());
+        }
+    }
 
-            if let Some(oneof) = field.desc.containing_oneof() {
-                for oneof_field in oneof.fields() {
-                    if oneof_field.number() != number {
-                        self.clear_field(oneof_field.number());
-                    }
-                }
+    fn clear_oneof_fields(&mut self, oneof_desc: OneofDescriptor, set_field: u32) {
+        for oneof_field in oneof_desc.fields() {
+            if oneof_field.number() != set_field {
+                self.clear_field(&oneof_field);
             }
         }
     }
 
-    /// Clears the field with number `number`.
+    /// Clears the given field.
     ///
     /// After calling this method, `has_field` will return false for the field,
     /// and it will not be included in the encoded bytes of this message.
+    pub fn clear_field(&mut self, field_desc: &FieldDescriptor) {
+        self.fields.clear(field_desc);
+    }
+
+    /// Returns `true` if this message has a field set with the given number.
     ///
-    /// If no field has number `number` this method does nothing.
-    pub fn clear_field(&mut self, number: u32) {
-        if let Some(field) = self.fields.get_mut(&number) {
-            field.clear();
+    /// See [`has_field`][Self::has_field] for more details.
+    pub fn has_field_by_number(&self, number: u32) -> bool {
+        self.desc
+            .get_field(number)
+            .map_or(false, |field_desc| self.has_field(&field_desc))
+    }
+
+    /// Gets the value of the field with the given number, or the default value if it is unset.
+    ///
+    /// If the message has no field with the given number, `None` is returned.
+    ///
+    /// See [`get_field`][Self::get_field] for more details.
+    pub fn get_field_by_number(&self, number: u32) -> Option<Cow<'_, Value>> {
+        self.desc
+            .get_field(number)
+            .map(|field_desc| self.get_field(&field_desc))
+    }
+
+    /// Sets the value of the field with number `number`, or the default value if it is unset.
+    ///
+    /// If no field with the given number exists, this method does nothing.
+    ///
+    /// See [`set_field`][Self::set_field] for more details.
+    pub fn set_field_by_number(&mut self, number: u32, value: Value) {
+        if let Some(field_desc) = self.desc.get_field(number) {
+            self.set_field(&field_desc, value)
         }
     }
 
-    /// Returns `true` if this message has a field set with the number `number`.
+    /// Clears the field with the given number.
+    ///
+    /// If no field with the given number exists, this method does nothing.
+    ///
+    /// See [`clear_field`][Self::clear_field] for more details.
+    pub fn clear_field_by_number(&mut self, number: u32) {
+        if let Some(field_desc) = self.desc.get_field(number) {
+            self.clear_field(&field_desc);
+        }
+    }
+
+    /// Returns `true` if this message has a field set with the given name.
     ///
     /// See [`has_field`][Self::has_field] for more details.
     pub fn has_field_by_name(&self, name: &str) -> bool {
         self.desc
             .get_field_by_name(name)
-            .map_or(false, |field_desc| self.has_field(field_desc.number()))
+            .map_or(false, |field_desc| self.has_field(&field_desc))
     }
 
-    /// Gets the value of the field with name `name`, or the default value if it is unset.
+    /// Gets the value of the field with the given name, or the default value if it is unset.
+    ///
+    /// If the message has no field with the given name, `None` is returned.
     ///
     /// See [`get_field`][Self::get_field] for more details.
     pub fn get_field_by_name(&self, name: &str) -> Option<Cow<'_, Value>> {
         self.desc
             .get_field_by_name(name)
-            .map(|field_desc| self.get_field(field_desc.number()).expect("field not set"))
+            .map(|field_desc| self.get_field(&field_desc))
     }
 
     /// Sets the value of the field with name `name`, or the default value if it is unset.
     ///
+    /// If no field with the given name exists, this method does nothing.
+    ///
     /// See [`set_field`][Self::set_field] for more details.
     pub fn set_field_by_name(&mut self, name: &str, value: Value) {
         if let Some(field_desc) = self.desc.get_field_by_name(name) {
-            self.set_field(field_desc.number(), value)
+            self.set_field(&field_desc, value)
         }
     }
 
-    /// Clears the field with name `name`.
+    /// Clears the field with the given name.
+    ///
+    /// If no field with the given name exists, this method does nothing.
     ///
     /// See [`clear_field`][Self::clear_field] for more details.
     pub fn clear_field_by_name(&mut self, name: &str) {
         if let Some(field_desc) = self.desc.get_field_by_name(name) {
-            self.clear_field(field_desc.number());
+            self.clear_field(&field_desc);
+        }
+    }
+
+    /// Returns `true` if this message has the given extension field set.
+    ///
+    /// See [`has_field`][Self::has_field] for more details.
+    pub fn has_extension(&self, extension_desc: &ExtensionDescriptor) -> bool {
+        self.extension_fields()
+            .map(|extensions| extensions.has(extension_desc))
+            .unwrap_or(false)
+    }
+
+    /// Gets the value of the give extension field, or the default value if it is unset.
+    ///
+    /// See [`get_field`][Self::get_field] for more details.
+    pub fn get_extension(&self, extension_desc: &ExtensionDescriptor) -> Option<Cow<'_, Value>> {
+        match self.extension_fields() {
+            Some(extensions) => Some(extensions.get(extension_desc)),
+            None => Some(Cow::Owned(Value::default_value_for_extension(
+                extension_desc,
+            ))),
+        }
+    }
+
+    /// Sets the value of the given extension field, or the default value if it is unset.
+    ///
+    /// See [`clear_field`][Self::clear_field] for more details.
+    pub fn set_extension(&mut self, extension_desc: &ExtensionDescriptor, value: Value) {
+        self.extension_fields_mut().set(extension_desc, value)
+    }
+
+    /// Clears the given extension field.
+    ///
+    /// See [`clear_field`][Self::clear_field] for more details.
+    pub fn clear_extension(&mut self, extension_desc: &ExtensionDescriptor) {
+        if let Some(cold) = &mut self.cold {
+            cold.extensions.clear(extension_desc)
         }
     }
 
@@ -210,6 +302,38 @@ impl DynamicMessage {
         let buf = self.encode_to_vec();
         T::decode(buf.as_slice())
     }
+
+    fn unknown_fields(&self) -> Option<&UnknownFieldSet> {
+        self.cold.as_ref().map(|c| &c.unknown)
+    }
+
+    fn extension_fields(&self) -> Option<&DynamicMessageFieldSet<ExtensionDescriptor>> {
+        self.cold.as_ref().map(|c| &c.extensions)
+    }
+
+    fn unknown_fields_mut(&mut self) -> &mut UnknownFieldSet {
+        &mut self.cold.get_or_insert_with(Default::default).unknown
+    }
+
+    fn extension_fields_mut(&mut self) -> &mut DynamicMessageFieldSet<ExtensionDescriptor> {
+        &mut self.cold.get_or_insert_with(Default::default).extensions
+    }
+}
+
+impl PartialEq for DynamicMessage {
+    fn eq(&self, other: &Self) -> bool {
+        self.desc == other.desc
+            && self.fields == other.fields
+            && match (&self.cold, &other.cold) {
+                (None, None) => true,
+                (None, Some(cold)) | (Some(cold), None) => {
+                    cold.unknown.is_empty() && cold.extensions.is_empty()
+                }
+                (Some(lhs), Some(rhs)) => {
+                    lhs.unknown == rhs.unknown && lhs.extensions == rhs.extensions
+                }
+            }
+    }
 }
 
 impl ReflectMessage for DynamicMessage {
@@ -218,56 +342,6 @@ impl ReflectMessage for DynamicMessage {
     }
 }
 
-impl DynamicMessageField {
-    pub fn new(desc: FieldDescriptor) -> Self {
-        DynamicMessageField {
-            value: if desc.supports_presence() {
-                None
-            } else {
-                Some(Value::default_value_for_field(&desc))
-            },
-            desc,
-        }
-    }
-
-    pub fn get(&self) -> Cow<'_, Value> {
-        match &self.value {
-            Some(value) => Cow::Borrowed(value),
-            None => Cow::Owned(Value::default_value_for_field(&self.desc)),
-        }
-    }
-
-    pub fn is_populated(&self) -> bool {
-        if self.desc.supports_presence() {
-            self.value.is_some()
-        } else {
-            !self
-                .value
-                .as_ref()
-                .unwrap()
-                .is_default_for_field(&self.desc)
-        }
-    }
-
-    pub fn set(&mut self, value: Value) {
-        debug_assert!(
-            value.is_valid_for_field(&self.desc),
-            "invalid value {:?} for field {:?}",
-            value,
-            self.desc,
-        );
-        self.value = Some(value);
-    }
-
-    pub fn clear(&mut self) {
-        self.value = if self.desc.supports_presence() {
-            None
-        } else {
-            Some(Value::default_value_for_field(&self.desc))
-        };
-    }
-}
-
 impl Value {
     /// Returns the default value for the given protobuf field.
     ///
@@ -288,6 +362,21 @@ impl Value {
         }
     }
 
+    /// Returns the default value for the given protobuf extension field.
+    ///
+    /// See [default_value_for_field][Value::default_value_for_field] for more details.
+    pub fn default_value_for_extension(extension_desc: &ExtensionDescriptor) -> Self {
+        if extension_desc.is_list() {
+            Value::List(Vec::default())
+        } else if extension_desc.is_map() {
+            Value::Map(HashMap::default())
+        } else if let Some(default_value) = extension_desc.default_value() {
+            default_value.clone()
+        } else {
+            Self::default_value(&extension_desc.kind())
+        }
+    }
+
     /// Returns the default value for the given protobuf type `kind`.
     pub fn default_value(kind: &Kind) -> Self {
         match kind {
@@ -310,6 +399,11 @@ impl Value {
         *self == Value::default_value_for_field(field_desc)
     }
 
+    /// Returns `true` if this is the default value for the given protobuf extension field.
+    pub fn is_default_for_extension(&self, extension_desc: &ExtensionDescriptor) -> bool {
+        *self == Value::default_value_for_extension(extension_desc)
+    }
+
     /// Returns `true` if this is the default value for the given protobuf type `kind`.
     pub fn is_default(&self, kind: &Kind) -> bool {
         *self == Value::default_value(kind)
@@ -335,6 +429,25 @@ impl Value {
         }
     }
 
+    /// Returns `true` if this value can be set for a given extension field.
+    ///
+    /// See [is_valid_for_field][Value::is_valid_for_field] for more details.
+    pub fn is_valid_for_extension(&self, extension_desc: &ExtensionDescriptor) -> bool {
+        match (self, extension_desc.kind()) {
+            (Value::List(list), kind) if extension_desc.is_list() => {
+                list.iter().all(|value| value.is_valid(&kind))
+            }
+            (Value::Map(map), Kind::Message(message_desc)) if extension_desc.is_map() => {
+                let key_desc = message_desc.map_entry_key_field().kind();
+                let value_desc = message_desc.map_entry_value_field();
+                map.iter().all(|(key, value)| {
+                    key.is_valid(&key_desc) && value.is_valid_for_field(&value_desc)
+                })
+            }
+            (value, kind) => value.is_valid(&kind),
+        }
+    }
+
     /// Returns `true` if this value can be encoded as the given [`Kind`].
     ///
     /// Unlike [`is_valid_for_field`](Value::is_valid_for_field), this method does not
